#! /usr/bin/octave 
# GA.m - Genetic Algroithm Interface Script
# Add on modules: import.m, curve.m, importfile.m, export.m, mutate.m
# Note: Main GA routines based on GAcontinuous.m by Haupt & Haupt (2003)

# Copyright 2009,2010  Richard James MacPherson, Grant Ingram and Jamie McIntosh
# This file is part of PEW.

#    PEW is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    PEW is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with PEW.  If not, see <http://www.gnu.org/licenses/>.


#####################
# Revision History - Jamie McIntosh
# 1.01 - Globals declared first and then values assigned
#        to remove bug not allowing the user to change parameters
# 1.02 - npar fixed to contain correct number of parameters
# 1.03 - Random cycle modified to assign values correctly. Scaling
#        has been fixed and is now only run in curve.m
# 1.04 - Added global user_id to allow connection to hamilton from
#        outside university network
# 1.05 - Method of scaling changed to scale correctly for any harmonic
#        series. par matrix adapted to include an 'amplitude' term
# 1.06 - Database link created - previous runs are now used as a basis
#        for future computations based on their similarity
# 1.07 - Database feature completed - succesful runs are now exported
# 1.08 - Added variable to allow multiple jobs to be batched when cluster
#        demand is high
# 1.09 - GA stopping criteria output displayed correctly
# 1.10 - User decides whether to use database or not
# 1.11 - Loading results from a previous run no longer clears the minc and
#        meanc values
# 1.12 - Hybrid selection scheme integrated. The number of stochastic selections
#        performed is determined by hybrid_routine set in GA.m
#
#####################
# Known Issues
# 1. Mutation rate is excessive - consider modifying
# 2. Needs to be given execute privileges and run from a terminal to work 
#    as intended. Also elevated status must be used in order for bash scripts
#    in importfile.m to work.
#
#####################

ignore_function_time_stamp ("all");  % Improves octave speed

clear all;                           % Clearing the current workspace
disp('GA.m - Genetic Algorithm Design of PEW')
rand('twister',sum(100*clock))       % Initialise Random Number Generator

# Cascade parameters
global pitch; global base_dir; global cluster_dir; global user_id;

pitch=191; 
base_dir="/home/jamie/Desktop/grant/"; % working directory 
cluster_dir="/data/hamilton/d61eed/pew/";      % directory on cluster
user_id="d61eed";                              % ITS username

addpath(base_dir);

%% Note that on the cluster you need a 'Blade Data' directory containing
%% lots of 'Blade Data' files and inlet boundary layer profile file

# GA Parameters - Initialising the Genetic Algroithm
global iga; global par;

# Parameterising the Endwall Surface
global harmonics; global constantshi; global constantslo;
global heighthi; global heightlow;

harmonics=1;              	     % Number of Fourier Series harmonics
constantshi=1; constantslo=-1;       % Fourier Series Coefficient limits
heighthi=20; heightlow=0;            % Pertubation height variable limits

# Stopping criteria and performance
global maxit; global mincost;

maxit=2;    % Max number of iterations
mincost=0;    % Minimum cost
global minc;  % minimum cost for each generation
global meanc; % mean cost for each generation
global cost;  % array for costs of everything!

# GA parameters
global popsize; global mutrate; global selection; global nbatch; global hybrid_routine;
  
popsize=24;       % Population Size
nbatch = 1;       % Number of chromosomes to be run in each jobfile
mutrate=0.2;      % Mutation Rate
selection=0.5;    % Fraction of population kept in Natural Selection
hybrid_routine=1; % Perform elitist selection on and after this generation
                  % Stochastic selection performed otherwise

# Calculated Variables 
global npar; global Nt; global keep; global nmut; 

npar=((harmonics*2)+1)*6;          % Number of Variables in Geometry Matrix
Nt=npar;                           % Continuous parameter GA Nt = No of variables
keep=floor(selection*popsize);     % No of population members that survive
nmut=ceil((popsize-1)*Nt*mutrate); % Total number of Mutations

# Starting the GA  

user_input = input('Load results from a previous run? y/n: ', 's');

if (user_input == 'y') % Load Results from previous run
  disp('Loading results from a previous run')
  iga = input('Which run to start from? (Enter Generation Number): ', 's');
  filename = ['Gen_' iga];
  load (filename)              %Loading Generation log file
  disp('Loaded variables')
else % start from scratch
  disp('Reseting GA')
  iga=0; % Initialise Generation Counter
  ## Populates geometry matrix with random numbers
  par=(constantshi-constantslo)*rand(popsize,npar)+constantslo; 
  ## Format Geometry Matrix for Pertubation Height
  for col=(npar/6):(npar/6):npar % Scales par scaling factor between 0 and maxperturb
    for cycle=1:1:popsize
      if par(cycle,col) < 0
	par(cycle,col)=(-1)*par(cycle,col); % Amplitude scaling must be +ve
      endif
      par(cycle,col)=((heighthi-heightlow)*par(cycle,col)+heightlow);
    endfor
  endfor 
endif

# Decide whether to use database or not
global datalink;
data_choose = input('Use database to determine interpolation of results (Selecting no engages space saving)? y/n: ', 's');
if (data_choose == 'y') % Make use of the database
  datalink = 1;
else
  datalink = 0;
endif

printf("*** Starting from Generation: %d ***\n",iga);
printf("Popsize: %d, No of parameters: %d \n",popsize,npar);

if user_input != 'y' % Only create a new log on the first generation
  ## Generate and save the system log 
  for a=1:1:popsize 
    log((((popsize*(iga)))+a),:)=par(a,:);
  endfor

  filename = ['Gen_' int2str(iga)];
  save(filename, 'log', 'iga', 'par')  %Saving Generation log 
endif

while iga<maxit # Iterate through generations
  iga=iga+1;    # Increments Generation Counter
  printf("\n*** Generation: %d ***\n",iga);

  if datalink == 1
    import   % Locate the most similar interpolation file 
  endif
  curve      % Generate endwall curves and submit to cluster 
  monitor    % Check the status of the submitted jobs on the cluster
  importfile % Reads in results and calculates cost function
  if datalink == 1
    export   % Exports successful designs to a local file store and database
  endif
  mutate     % Performs mutation

  ## Graphing the progression of the GA

  iters=0:length(minc)-1;
  plot(iters,minc,iters,meanc,'--');
  xlabel('generation');ylabel('cost');
  text(0,minc(1),'best');text(1,minc(2),'population average') ##JAMIE: should minc(2) not be meanc?

  ## GA Stopping Criteria

  if (iga>=maxit || cost(1)<mincost)
    printf("\nOptimal Solution Has Been Found!");
    break
  endif

endwhile
       
