## curve.m - Generates Fourier Series curves and Submits files to Hamilton
## Description: Generates Fourier Series Curves
##              Generates Hamilton Input Script
##              Generates Hamilton Submission Shell Script
##              Moves GAMBIT & FLUENT files into local folders
##              Submits files to Hamilton
## Notes: Before running check ALL file destinations
##        Also ssh keys need to be set up to allow pasword free login 

# Copyright 2009,2010  Richard James MacPherson, Grant Ingram and Jamie McIntosh
# This file is part of PEW.

#    PEW is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    PEW is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with PEW.  If not, see <http://www.gnu.org/licenses/>.


#####################
# Revision History - Jamie McIntosh
# 1.01 - Fourier curve generation fixed for yval, now uses correct
#        parameters in each chromosome
# 1.02 - Scaling fixed and first generation created as intended
# 1.03 - ssh connection now possible from outside of university network
# 1.04 - Scaling method changed and now final. Working as intended for all
#        lengths of fourier series
# 1.05 - Survivors of each generation are copied into the next Generation
#        filestore so that repeat testing is removed and script modified to
#        skip these chromosomes in CFD analysis
# 1.06 - Reverted to 1.04 - skipping affects mutation
# 1.07 - Submit scripts modified to allow batching multiple jobs when cluster
#        load is high
# 1.08 - Curve modified to use devmyri.q instead of seq.q on cluster
# 1.09 - Integrated database choice option and space saver
# 1.10 - Vega script added to process fvuns file using fieldview
# 1.11 - Non intensive jobs, ie server up/downloading now performed on vega
#
#####################
# Known Issues
#
#####################

function curve() % generates fourier series curves and input files 

global iga; global harmonics; global popsize; global npar;
global heighthi; global heightlow; global par; global pitch;
global base_dir; global cluster_dir; global user_id;
global job_id; global keep; global nbatch; global datalink;

## Location of Axial Planes and Curve Starting Vertex Locations
axial=[-204;-164.32;-126.4;-86.67;-48.75;-9.03];
startpoint=[217.82; 253.91; 270.29; 257.46; 208.6; 114.87];

##  Generates the Generation File in the Current Working Directory 
newdir = sprintf("Generation_%d", iga); 
mkdir(newdir); cd(newdir);
disp(sprintf("Generation %d",iga));

## Generates the Chromosome File & Curve Files 

for chrom=1:1:popsize                % Cycle through chromosomes
  newdir = sprintf("chr_%d", chrom); % Generates the chromosome file
  mkdir(newdir);
  cd(newdir);

  for line=0:1:5          % Cycle through axial planes
    xval = 0:0.01:(2*pi); % Initialise xvalue in range (0 - 2pi) at 0.01 radians
    yval = 0; ymax = 0;   % Initialise y and ymax values
 
    for harm=1:1:(harmonics) % Generates Fourier series curve
            yval = yval + ((par(chrom,(harm+(((harmonics*2)+1)*line))))*sin(harm*xval)) + ((par(chrom,((harm+harmonics)+(((harmonics*2)+1)*line))))*cos(harm*xval));
    end
    
    ymax = max(abs(max(yval)), abs(min(yval))); % Finds maximum value for scaling

    ## Scales the Fourier series in relation to maximum pertubation height  
    yval = yval*(par(chrom,((npar/6)+((npar/6)*line)))/ymax); % Scales line between -1 & 1 and 
							      % then scaled by the par amplitude

    ## Scales the curves to fit within domain
    xval = xval*pitch/(2*pi);
    xval = xval + (startpoint(line+1)-pitch);  ##JAMIE: What is startpoint?

    ## Generates files to store Fourier Series Curves

    filename = ['line_' int2str(line+1) '.csv'];
    [fid] = fopen(filename, 'w');
       for o=1:1:629
         fprintf(fid,'%3.6f', axial(line+1));
         fprintf(fid,'%s',',');
         fprintf(fid,'%3.6f', xval(o));
         fprintf(fid,'%s',',');
         fprintf(fid,'%2.6f\n', yval(o));
       end
    fclose(fid);
  end
  disp(sprintf("Created Endwall Curve Files for chr %d",chrom));

  ## Copies GAMBIT & FLUENT Files from current working directory to chr directory
  sy1 = sprintf("cp %smesh.jou .",base_dir);
  system(sy1);
 
  if datalink == 0 % don't use database - space saving
    sy2 = sprintf("cp %sinputfile1 ./inputfile",base_dir); % Planar Case, space saving mode
  else
    sy2 = sprintf("cp %sinputfile2 ./inputfile",base_dir); % Most Similar Case
  endif
  system(sy2);

  sy3 = sprintf("cp %sfv* .",base_dir); % Fieldview Streamline Post-Processing Files
  system(sy3);

  ## Generate Script that runs Post-Processing on Vega
  filename = ['fv_process.csh'];
  [fid] = fopen(filename, 'w');
  fprintf(fid,"#!/bin/csh \n");
  fprintf(fid,"# %s script generated by curve.m \n",filename);
  fprintf(fid,"ssh -Y %s@vega3.dur.ac.uk ",user_id);
  fprintf(fid,"'"); % open the command list in the script
  fprintf(fid,"cd %sGeneration_%d/chr_%d/ ;",cluster_dir,iga,chrom)
  fprintf(fid,"fv -batch -fvx fvprocess.fvx >& fv.out");
  fprintf(fid,"'"); % close the command list in the script
  fclose(fid);

  cd('..'); % end of chromosomes
end

## Process Population on Hamilton 
batchid = 0;
chrom_count = 0;
while(chrom_count<popsize)
  batchid++; % Current batch number
  ## Generate Hamilton INPUT File
  filename = ['Gen_' int2str(iga) 'chrs_' int2str(chrom_count+1) '-' int2str(chrom_count+nbatch) '.csh'];
  [fid] = fopen(filename, 'w');
  fprintf(fid,"#!/bin/csh \n");
  fprintf(fid,"# %s script generated by curve.m \n",filename);
  fprintf(fid,"# use current working directory \n");
  fprintf(fid,"#$ -cwd \n\n");
  fprintf(fid,"# combine standard error and output files \n");
  fprintf(fid,"#$ -j y \n\n");
  fprintf(fid,"# request submission to a queue for serial jobs \n");
  fprintf(fid,"#$ -q par.q \n\n"); 
  fprintf(fid,"# execute commands \n");
  fprintf(fid,"module add fluent/current \n"); 
  i=0; % for loop counter
  for batch=1:1:nbatch
    chrom_count++;
    i++;
    if chrom_count > popsize
      break % Stops loop when population complete
    endif   
    fprintf(fid,"cd chr_%d \n",chrom_count);
    fprintf(fid,"bash gambit -inp mesh.jou >& gambit.out \n");            
    fprintf(fid,"fluent 3d -i inputfile -g -sge >& fluent.out \n");
    fprintf(fid,"cd .. \n\n");
  endfor
  fclose(fid);
  chrom_count = chrom_count - i; % return counter to correct start value
  		
  ## Generate submit script for Hamilton
  marker = ['submit_' int2str(batchid)];
  [fid] = fopen(marker, 'w');
  fprintf(fid,"#!/bin/bash \n");
  for batch=1:1:nbatch
    chrom_count++;
    if chrom_count > popsize
      break % Stops loop when population complete
    endif  
    fprintf(fid,"ssh %s@vega3.dur.ac.uk 'mkdir -p %sGeneration_%d/chr_%d/'\n"
  	    ,user_id,cluster_dir,iga,chrom_count);
    fprintf(fid,"scp %sGeneration_%d/chr_%d/* %s@vega3.dur.ac.uk:%sGeneration_%d/chr_%d/\n"
	    ,base_dir,iga,chrom_count,user_id,cluster_dir,iga,chrom_count);
  endfor
  fprintf(fid,"ssh %s@hamilton.dur.ac.uk ",user_id);
  fprintf(fid,"'"); % open the command list in the script
  fprintf(fid,"setenv SGE_ROOT /usr/local/Cluster-Apps/sge ;");
  fprintf(fid,"cd %sGeneration_%d/ ;",cluster_dir,iga);
  fprintf(fid,"/usr/local/Cluster-Apps/sge/bin/lx26-amd64/qsub -q par.q %sGeneration_%d/Gen_%dchrs_%d-%d.csh",cluster_dir,iga,iga,chrom_count-i+1,chrom_count-i+nbatch);
  fprintf(fid,"'"); % close the command list in the script
  fclose(fid);

endwhile

## Submit Jobs to Hamilton
sy4 = sprintf("ssh %s@vega3.dur.ac.uk 'mkdir -p %sGeneration_%d/'",user_id,cluster_dir,iga);
system(sy4); % create generation filestore fpr script files
sy5 = sprintf("scp -pq Gen_* %s@vega3.dur.ac.uk:%sGeneration_%d/",user_id,cluster_dir,iga);
system(sy5); % copy script files to hamilton 
for batch=1:1:batchid
  sy6 = sprintf("bash submit_%d",batch);
  [status, result]=system(sy6);

  ## Logs Hamilton job_id number
  if status == 0
	  idx = regexp(result,'\d+');
	  nums = regexp(result,'\d+','match'); 
	  job=str2num(nums{1});
	  job_id(batch,1)=job;
	  disp(['Submitted job, job_id number:' int2str(job)]);
  endif
endfor

cd('..'); % end of generation
endfunction
